@@ -5,17 +5,19 @@ import math
 import os.path
 import sys
 import tkinter as tk
-from tkinter.simpledialog import SimpleDialog
 import tkinter.filedialog
+import tkinter.font
 import tkinter.messagebox
+from tkinter.simpledialog import SimpleDialog
 
 import numpy as np
+from PIL import Image, ImageTk
 
 import matplotlib as mpl
 from matplotlib import _api, backend_tools, cbook, _c_internal_utils
 from matplotlib.backend_bases import (
     _Backend, FigureCanvasBase, FigureManagerBase, NavigationToolbar2,
-    StatusbarBase, TimerBase, ToolContainerBase, cursors, _Mode)
+    TimerBase, ToolContainerBase, cursors, _Mode)
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.figure import Figure
 from matplotlib.widgets import SubplotTool
@@ -32,7 +34,9 @@ cursord = {
     cursors.POINTER: "arrow",
     cursors.SELECT_REGION: "tcross",
     cursors.WAIT: "watch",
-    }
+    cursors.RESIZE_HORIZONTAL: "sb_h_double_arrow",
+    cursors.RESIZE_VERTICAL: "sb_v_double_arrow",
+}
 
 
 @contextmanager
@@ -49,6 +53,9 @@ _blit_args = {}
 # Initialize to a non-empty string that is not a Tcl command
 _blit_tcl_name = "mpl_blit_" + uuid.uuid4().hex
 
+TK_PHOTO_COMPOSITE_OVERLAY = 0  # apply transparency rules pixel-wise
+TK_PHOTO_COMPOSITE_SET = 1  # set image buffer directly
+
 
 def _blit(argsid):
     """
@@ -56,15 +63,10 @@ def _blit(argsid):
 
     *argsid* is a unique string identifier to fetch the correct arguments from
     the ``_blit_args`` dict, since arguments cannot be passed directly.
-
-    photoimage blanking must occur in the same event and thread as blitting
-    to avoid flickering.
     """
-    photoimage, dataptr, offsets, bboxptr, blank = _blit_args.pop(argsid)
-    if blank:
-        photoimage.blank()
-    _tkagg.blit(
-        photoimage.tk.interpaddr(), str(photoimage), dataptr, offsets, bboxptr)
+    photoimage, dataptr, offsets, bboxptr, comp_rule = _blit_args.pop(argsid)
+    _tkagg.blit(photoimage.tk.interpaddr(), str(photoimage), dataptr,
+                comp_rule, offsets, bboxptr)
 
 
 def blit(photoimage, aggimage, offsets, bbox=None):
@@ -77,7 +79,7 @@ def blit(photoimage, aggimage, offsets, bbox=None):
     for big-endian ARGB32 (i.e. ARGB8888) data.
 
     If *bbox* is passed, it defines the region that gets blitted. That region
-    will NOT be blanked before blitting.
+    will be composed with the previous data according to the alpha channel.
 
     Tcl events must be dispatched to trigger a blit from a non-Tcl thread.
     """
@@ -91,10 +93,10 @@ def blit(photoimage, aggimage, offsets, bbox=None):
         y1 = max(math.floor(y1), 0)
         y2 = min(math.ceil(y2), height)
         bboxptr = (x1, x2, y1, y2)
-        blank = False
+        comp_rule = TK_PHOTO_COMPOSITE_OVERLAY
     else:
         bboxptr = (0, width, 0, height)
-        blank = True
+        comp_rule = TK_PHOTO_COMPOSITE_SET
 
     # NOTE: _tkagg.blit is thread unsafe and will crash the process if called
     # from a thread (GH#13293). Instead of blanking and blitting here,
@@ -103,7 +105,7 @@ def blit(photoimage, aggimage, offsets, bbox=None):
 
     # tkapp.call coerces all arguments to strings, so to avoid string parsing
     # within _blit, pack up the arguments into a global data structure.
-    args = photoimage, dataptr, offsets, bboxptr, blank
+    args = photoimage, dataptr, offsets, bboxptr, comp_rule
     # Need a unique key to avoid thread races.
     # Again, make the key a string to avoid string parsing in _blit.
     argsid = str(id(args))
@@ -164,9 +166,9 @@ class FigureCanvasTk(FigureCanvasBase):
         alternative="get_tk_widget().bind('<Configure>', ..., True)")
     def __init__(self, figure=None, master=None, resize_callback=None):
         super().__init__(figure)
-        self._idle = True
-        self._idle_callback = None
-        w, h = self.figure.bbox.size.astype(int)
+        self._idle_draw_id = None
+        self._event_loop_id = None
+        w, h = self.get_width_height(physical=True)
         self._tkcanvas = tk.Canvas(
             master=master, background="white",
             width=w, height=h, borderwidth=0, highlightthickness=0)
@@ -175,6 +177,8 @@ class FigureCanvasTk(FigureCanvasBase):
         self._tkcanvas.create_image(w//2, h//2, image=self._tkphoto)
         self._resize_callback = resize_callback
         self._tkcanvas.bind("<Configure>", self.resize)
+        if sys.platform == 'win32':
+            self._tkcanvas.bind("<Map>", self._update_device_pixel_ratio)
         self._tkcanvas.bind("<Key>", self.key_press)
         self._tkcanvas.bind("<Motion>", self.motion_notify_event)
         self._tkcanvas.bind("<Enter>", self.enter_notify_event)
@@ -195,7 +199,7 @@ class FigureCanvasTk(FigureCanvasBase):
         # Mouse wheel for windows goes to the window with the focus.
         # Since the canvas won't usually have the focus, bind the
         # event to the window containing the canvas instead.
-        # See http://wiki.tcl.tk/3893 (mousewheel) for details
+        # See https://wiki.tcl-lang.org/3893 (mousewheel) for details
         root = self._tkcanvas.winfo_toplevel()
         root.bind("<MouseWheel>", self.scroll_event_windows, "+")
 
@@ -206,9 +210,20 @@ class FigureCanvasTk(FigureCanvasBase):
                 self.close_event()
         root.bind("<Destroy>", filter_destroy, "+")
 
-        self._master = master
         self._tkcanvas.focus_set()
 
+    def _update_device_pixel_ratio(self, event=None):
+        # Tk gives scaling with respect to 72 DPI, but Windows screens are
+        # scaled vs 96 dpi, and pixel ratio settings are given in whole
+        # percentages, so round to 2 digits.
+        ratio = round(self._tkcanvas.tk.call('tk', 'scaling') / (96 / 72), 2)
+        if self._set_device_pixel_ratio(ratio):
+            # The easiest way to resize the canvas is to resize the canvas
+            # widget itself, since we implement all the logic for resizing the
+            # canvas backing store on that event.
+            w, h = self.get_width_height(physical=True)
+            self._tkcanvas.configure(width=w, height=h)
+
     def resize(self, event):
         width, height = event.width, event.height
         if self._resize_callback is not None:
@@ -229,18 +244,16 @@ class FigureCanvasTk(FigureCanvasBase):
 
     def draw_idle(self):
         # docstring inherited
-        if not self._idle:
+        if self._idle_draw_id:
             return
 
-        self._idle = False
-
         def idle_draw(*args):
             try:
                 self.draw()
             finally:
-                self._idle = True
+                self._idle_draw_id = None
 
-        self._idle_callback = self._tkcanvas.after_idle(idle_draw)
+        self._idle_draw_id = self._tkcanvas.after_idle(idle_draw)
 
     def get_tk_widget(self):
         """
@@ -251,68 +264,56 @@ class FigureCanvasTk(FigureCanvasBase):
         """
         return self._tkcanvas
 
+    def _event_mpl_coords(self, event):
+        # calling canvasx/canvasy allows taking scrollbars into account (i.e.
+        # the top of the widget may have been scrolled out of view).
+        return (self._tkcanvas.canvasx(event.x),
+                # flipy so y=0 is bottom of canvas
+                self.figure.bbox.height - self._tkcanvas.canvasy(event.y))
+
     def motion_notify_event(self, event):
-        x = event.x
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y
-        super().motion_notify_event(x, y, guiEvent=event)
+        super().motion_notify_event(
+            *self._event_mpl_coords(event), guiEvent=event)
 
     def enter_notify_event(self, event):
-        x = event.x
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y
-        super().enter_notify_event(guiEvent=event, xy=(x, y))
+        super().enter_notify_event(
+            guiEvent=event, xy=self._event_mpl_coords(event))
 
     def button_press_event(self, event, dblclick=False):
-        x = event.x
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y
+        # set focus to the canvas so that it can receive keyboard events
+        self._tkcanvas.focus_set()
+
         num = getattr(event, 'num', None)
-
-        if sys.platform == 'darwin':
-            # 2 and 3 were reversed on the OSX platform I tested under tkagg.
-            if num == 2:
-                num = 3
-            elif num == 3:
-                num = 2
-
-        super().button_press_event(x, y, num,
-                                   dblclick=dblclick, guiEvent=event)
+        if sys.platform == 'darwin':  # 2 and 3 are reversed.
+            num = {2: 3, 3: 2}.get(num, num)
+        super().button_press_event(
+            *self._event_mpl_coords(event), num, dblclick=dblclick,
+            guiEvent=event)
 
     def button_dblclick_event(self, event):
         self.button_press_event(event, dblclick=True)
 
     def button_release_event(self, event):
-        x = event.x
-        # flipy so y=0 is bottom of canvas
-        y = self.figure.bbox.height - event.y
-
         num = getattr(event, 'num', None)
-
-        if sys.platform == 'darwin':
-            # 2 and 3 were reversed on the OSX platform I tested under tkagg.
-            if num == 2:
-                num = 3
-            elif num == 3:
-                num = 2
-
-        super().button_release_event(x, y, num, guiEvent=event)
+        if sys.platform == 'darwin':  # 2 and 3 are reversed.
+            num = {2: 3, 3: 2}.get(num, num)
+        super().button_release_event(
+            *self._event_mpl_coords(event), num, guiEvent=event)
 
     def scroll_event(self, event):
-        x = event.x
-        y = self.figure.bbox.height - event.y
         num = getattr(event, 'num', None)
         step = 1 if num == 4 else -1 if num == 5 else 0
-        super().scroll_event(x, y, step, guiEvent=event)
+        super().scroll_event(
+            *self._event_mpl_coords(event), step, guiEvent=event)
 
     def scroll_event_windows(self, event):
         """MouseWheel event processor"""
         # need to find the window that contains the mouse
         w = event.widget.winfo_containing(event.x_root, event.y_root)
         if w == self._tkcanvas:
-            x = event.x_root - w.winfo_rootx()
-            y = event.y_root - w.winfo_rooty()
-            y = self.figure.bbox.height - y
+            x = self._tkcanvas.canvasx(event.x_root - w.winfo_rootx())
+            y = (self.figure.bbox.height
+                 - self._tkcanvas.canvasy(event.y_root - w.winfo_rooty()))
             step = event.delta/120.
             FigureCanvasBase.scroll_event(self, x, y, step, guiEvent=event)
 
@@ -369,17 +370,32 @@ class FigureCanvasTk(FigureCanvasBase):
 
     def flush_events(self):
         # docstring inherited
-        self._master.update()
+        self._tkcanvas.update()
 
     def start_event_loop(self, timeout=0):
         # docstring inherited
         if timeout > 0:
-            self._master.after(int(1000*timeout), self.stop_event_loop)
-        self._master.mainloop()
+            milliseconds = int(1000 * timeout)
+            if milliseconds > 0:
+                self._event_loop_id = self._tkcanvas.after(
+                    milliseconds, self.stop_event_loop)
+            else:
+                self._event_loop_id = self._tkcanvas.after_idle(
+                    self.stop_event_loop)
+        self._tkcanvas.mainloop()
 
     def stop_event_loop(self):
         # docstring inherited
-        self._master.quit()
+        if self._event_loop_id:
+            self._tkcanvas.after_cancel(self._event_loop_id)
+            self._event_loop_id = None
+        self._tkcanvas.quit()
+
+    def set_cursor(self, cursor):
+        try:
+            self._tkcanvas.configure(cursor=cursord[cursor])
+        except tkinter.TclError:
+            pass
 
 
 class FigureManagerTk(FigureManagerBase):
@@ -412,6 +428,17 @@ class FigureManagerTk(FigureManagerBase):
             if self.toolbar:
                 backend_tools.add_tools_to_container(self.toolbar)
 
+        # If the window has per-monitor DPI awareness, then setup a Tk variable
+        # to store the DPI, which will be updated by the C code, and the trace
+        # will handle it on the Python side.
+        window_frame = int(window.wm_frame(), 16)
+        self._window_dpi = tk.IntVar(master=window, value=96,
+                                     name=f'window_dpi{window_frame}')
+        self._window_dpi_cbname = ''
+        if _tkagg.enable_dpi_awareness(window_frame, window.tk.interpaddr()):
+            self._window_dpi_cbname = self._window_dpi.trace_add(
+                'write', self._update_window_dpi)
+
         self._shown = False
 
     def _get_toolbar(self):
@@ -423,6 +450,13 @@ class FigureManagerTk(FigureManagerBase):
             toolbar = None
         return toolbar
 
+    def _update_window_dpi(self, *args):
+        newdpi = self._window_dpi.get()
+        self.window.call('tk', 'scaling', newdpi / 72)
+        if self.toolbar and hasattr(self.toolbar, '_rescale'):
+            self.toolbar._rescale()
+        self.canvas._update_device_pixel_ratio()
+
     def resize(self, width, height):
         max_size = 1_400_000  # the measured max on xorg 1.20.8 was 1_409_023
 
@@ -444,6 +478,7 @@ class FigureManagerTk(FigureManagerBase):
                     Gcf.destroy(self)
                 self.window.protocol("WM_DELETE_WINDOW", destroy)
                 self.window.deiconify()
+                self.canvas._tkcanvas.focus_set()
             else:
                 self.canvas.draw_idle()
             if mpl.rcParams['figure.raise_window']:
@@ -452,21 +487,30 @@ class FigureManagerTk(FigureManagerBase):
             self._shown = True
 
     def destroy(self, *args):
-        if self.canvas._idle_callback:
-            self.canvas._tkcanvas.after_cancel(self.canvas._idle_callback)
+        if self.canvas._idle_draw_id:
+            self.canvas._tkcanvas.after_cancel(self.canvas._idle_draw_id)
+        if self.canvas._event_loop_id:
+            self.canvas._tkcanvas.after_cancel(self.canvas._event_loop_id)
+        if self._window_dpi_cbname:
+            self._window_dpi.trace_remove('write', self._window_dpi_cbname)
 
         # NOTE: events need to be flushed before issuing destroy (GH #9956),
-        # however, self.window.update() can break user code. This is the
-        # safest way to achieve a complete draining of the event queue,
-        # but it may require users to update() on their own to execute the
-        # completion in obscure corner cases.
+        # however, self.window.update() can break user code. An async callback
+        # is the safest way to achieve a complete draining of the event queue,
+        # but it leaks if no tk event loop is running. Therefore we explicitly
+        # check for an event loop and choose our best guess.
         def delayed_destroy():
             self.window.destroy()
 
             if self._owns_mainloop and not Gcf.get_num_fig_managers():
                 self.window.quit()
 
-        self.window.after_idle(delayed_destroy)
+        if cbook._get_running_interactive_framework() == "tk":
+            # "after idle after 0" avoids Tcl error/race (GH #19940)
+            self.window.after_idle(self.window.after, 0, delayed_destroy)
+        else:
+            self.window.update()
+            delayed_destroy()
 
     def get_window_title(self):
         return self.window.wm_title()
@@ -516,22 +560,52 @@ class NavigationToolbar2Tk(NavigationToolbar2, tk.Frame):
                 if tooltip_text is not None:
                     ToolTip.createToolTip(button, tooltip_text)
 
+        self._label_font = tkinter.font.Font(root=window, size=10)
+
         # This filler item ensures the toolbar is always at least two text
         # lines high. Otherwise the canvas gets redrawn as the mouse hovers
         # over images because those use two-line messages which resize the
         # toolbar.
-        label = tk.Label(master=self,
+        label = tk.Label(master=self, font=self._label_font,
                          text='\N{NO-BREAK SPACE}\n\N{NO-BREAK SPACE}')
         label.pack(side=tk.RIGHT)
 
         self.message = tk.StringVar(master=self)
-        self._message_label = tk.Label(master=self, textvariable=self.message)
+        self._message_label = tk.Label(master=self, font=self._label_font,
+                                       textvariable=self.message)
         self._message_label.pack(side=tk.RIGHT)
 
         NavigationToolbar2.__init__(self, canvas)
         if pack_toolbar:
             self.pack(side=tk.BOTTOM, fill=tk.X)
 
+    def _rescale(self):
+        """
+        Scale all children of the toolbar to current DPI setting.
+
+        Before this is called, the Tk scaling setting will have been updated to
+        match the new DPI. Tk widgets do not update for changes to scaling, but
+        all measurements made after the change will match the new scaling. Thus
+        this function re-applies all the same sizes in points, which Tk will
+        scale correctly to pixels.
+        """
+        for widget in self.winfo_children():
+            if isinstance(widget, (tk.Button, tk.Checkbutton)):
+                if hasattr(widget, '_image_file'):
+                    # Explicit class because ToolbarTk calls _rescale.
+                    NavigationToolbar2Tk._set_image_for_button(self, widget)
+                else:
+                    # Text-only button is handled by the font setting instead.
+                    pass
+            elif isinstance(widget, tk.Frame):
+                widget.configure(height='22p', pady='1p')
+                widget.pack_configure(padx='4p')
+            elif isinstance(widget, tk.Label):
+                pass  # Text is handled by the font setting instead.
+            else:
+                _log.warning('Unknown child class %s', widget.winfo_class)
+        self._label_font.configure(size=10)
+
     def _update_buttons_checked(self):
         # sync button checkstates to match active mode
         for text, mode in [('Zoom', _Mode.ZOOM), ('Pan', _Mode.PAN)]:
@@ -553,35 +627,42 @@ class NavigationToolbar2Tk(NavigationToolbar2, tk.Frame):
         self.message.set(s)
 
     def draw_rubberband(self, event, x0, y0, x1, y1):
+        self.remove_rubberband()
         height = self.canvas.figure.bbox.height
         y0 = height - y0
         y1 = height - y1
-        if hasattr(self, "lastrect"):
-            self.canvas._tkcanvas.delete(self.lastrect)
         self.lastrect = self.canvas._tkcanvas.create_rectangle(x0, y0, x1, y1)
 
-    def release_zoom(self, event):
-        super().release_zoom(event)
+    def remove_rubberband(self):
         if hasattr(self, "lastrect"):
             self.canvas._tkcanvas.delete(self.lastrect)
             del self.lastrect
 
-    def set_cursor(self, cursor):
-        window = self.canvas.get_tk_widget().master
-        try:
-            window.configure(cursor=cursord[cursor])
-        except tkinter.TclError:
-            pass
+    def _set_image_for_button(self, button):
+        """
+        Set the image for a button based on its pixel size.
+
+        The pixel size is determined by the DPI scaling of the window.
+        """
+        if button._image_file is None:
+            return
+
+        # Allow _image_file to be relative to Matplotlib's "images" data
+        # directory.
+        path_regular = cbook._get_data_path('images', button._image_file)
+        path_large = path_regular.with_name(
+            path_regular.name.replace('.png', '_large.png'))
+        size = button.winfo_pixels('18p')
+        # Use the high-resolution (48x48 px) icon if it exists and is needed
+        with Image.open(path_large if (size > 24 and path_large.exists())
+                        else path_regular) as im:
+            image = ImageTk.PhotoImage(im.resize((size, size)), master=self)
+        button.configure(image=image, height='18p', width='18p')
+        button._ntimage = image  # Prevent garbage collection.
 
     def _Button(self, text, image_file, toggle, command):
-        if tk.TkVersion >= 8.6:
-            PhotoImage = tk.PhotoImage
-        else:
-            from PIL.ImageTk import PhotoImage
-        image = (PhotoImage(master=self, file=image_file)
-                 if image_file is not None else None)
         if not toggle:
-            b = tk.Button(master=self, text=text, image=image, command=command)
+            b = tk.Button(master=self, text=text, command=command)
         else:
             # There is a bug in tkinter included in some python 3.6 versions
             # that without this variable, produces a "visual" toggling of
@@ -590,18 +671,22 @@ class NavigationToolbar2Tk(NavigationToolbar2, tk.Frame):
             # https://bugs.python.org/issue25684
             var = tk.IntVar(master=self)
             b = tk.Checkbutton(
-                master=self, text=text, image=image, command=command,
+                master=self, text=text, command=command,
                 indicatoron=False, variable=var)
             b.var = var
-        b._ntimage = image
+        b._image_file = image_file
+        if image_file is not None:
+            # Explicit class because ToolbarTk calls _Button.
+            NavigationToolbar2Tk._set_image_for_button(self, b)
+        else:
+            b.configure(font=self._label_font)
         b.pack(side=tk.LEFT)
         return b
 
     def _Spacer(self):
-        # Buttons are 30px high. Make this 26px tall +2px padding to center it.
-        s = tk.Frame(
-            master=self, height=26, relief=tk.RIDGE, pady=2, bg="DarkGray")
-        s.pack(side=tk.LEFT, padx=5)
+        # Buttons are also 18pt high.
+        s = tk.Frame(master=self, height='18p', relief=tk.RIDGE, bg='DarkGray')
+        s.pack(side=tk.LEFT, padx='3p')
         return s
 
     def save_figure(self, *args):
@@ -619,7 +704,7 @@ class NavigationToolbar2Tk(NavigationToolbar2, tk.Frame):
         # asksaveasfilename dialog when you choose various save types
         # from the dropdown.  Passing in the empty string seems to
         # work - JDH!
-        #defaultextension = self.canvas.get_default_filetype()
+        # defaultextension = self.canvas.get_default_filetype()
         defaultextension = ''
         initialdir = os.path.expanduser(mpl.rcParams['savefig.directory'])
         initialfile = self.canvas.get_default_filename()
@@ -683,7 +768,7 @@ class ToolTip:
         if self.tipwindow or not self.text:
             return
         x, y, _, _ = self.widget.bbox("insert")
-        x = x + self.widget.winfo_rootx() + 27
+        x = x + self.widget.winfo_rootx() + self.widget.winfo_width()
         y = y + self.widget.winfo_rooty()
         self.tipwindow = tw = tk.Toplevel(self.widget)
         tw.wm_overrideredirect(1)
@@ -708,11 +793,10 @@ class ToolTip:
 
 class RubberbandTk(backend_tools.RubberbandBase):
     def draw_rubberband(self, x0, y0, x1, y1):
+        self.remove_rubberband()
         height = self.figure.canvas.figure.bbox.height
         y0 = height - y0
         y1 = height - y1
-        if hasattr(self, "lastrect"):
-            self.figure.canvas._tkcanvas.delete(self.lastrect)
         self.lastrect = self.figure.canvas._tkcanvas.create_rectangle(
             x0, y0, x1, y1)
 
@@ -722,6 +806,7 @@ class RubberbandTk(backend_tools.RubberbandBase):
             del self.lastrect
 
 
+@_api.deprecated("3.5", alternative="ToolSetCursor")
 class SetCursorTk(backend_tools.SetCursorBase):
     def set_cursor(self, cursor):
         NavigationToolbar2Tk.set_cursor(
@@ -736,18 +821,29 @@ class ToolbarTk(ToolContainerBase, tk.Frame):
         tk.Frame.__init__(self, master=window,
                           width=int(width), height=int(height),
                           borderwidth=2)
+        self._label_font = tkinter.font.Font(size=10)
         self._message = tk.StringVar(master=self)
-        self._message_label = tk.Label(master=self, textvariable=self._message)
+        self._message_label = tk.Label(master=self, font=self._label_font,
+                                       textvariable=self._message)
         self._message_label.pack(side=tk.RIGHT)
         self._toolitems = {}
         self.pack(side=tk.TOP, fill=tk.X)
         self._groups = {}
 
+    def _rescale(self):
+        return NavigationToolbar2Tk._rescale(self)
+
     def add_toolitem(
             self, name, group, position, image_file, description, toggle):
         frame = self._get_groupframe(group)
-        button = NavigationToolbar2Tk._Button(self, name, image_file, toggle,
+        buttons = frame.pack_slaves()
+        if position >= len(buttons) or position < 0:
+            before = None
+        else:
+            before = buttons[position]
+        button = NavigationToolbar2Tk._Button(frame, name, image_file, toggle,
                                               lambda: self._button_click(name))
+        button.pack_configure(before=before)
         if description is not None:
             ToolTip.createToolTip(button, description)
         self._toolitems.setdefault(name, [])
@@ -759,6 +855,7 @@ class ToolbarTk(ToolContainerBase, tk.Frame):
                 self._add_separator()
             frame = tk.Frame(master=self, borderwidth=0)
             frame.pack(side=tk.LEFT, fill=tk.Y)
+            frame._label_font = self._label_font
             self._groups[group] = frame
         return self._groups[group]
 
@@ -786,24 +883,6 @@ class ToolbarTk(ToolContainerBase, tk.Frame):
         self._message.set(s)
 
 
-@_api.deprecated("3.3")
-class StatusbarTk(StatusbarBase, tk.Frame):
-    def __init__(self, window, *args, **kwargs):
-        StatusbarBase.__init__(self, *args, **kwargs)
-        xmin, xmax = self.toolmanager.canvas.figure.bbox.intervalx
-        height, width = 50, xmax - xmin
-        tk.Frame.__init__(self, master=window,
-                          width=int(width), height=int(height),
-                          borderwidth=2)
-        self._message = tk.StringVar(master=self)
-        self._message_label = tk.Label(master=self, textvariable=self._message)
-        self._message_label.pack(side=tk.RIGHT)
-        self.pack(side=tk.TOP, fill=tk.X)
-
-    def set_message(self, s):
-        self._message.set(s)
-
-
 class SaveFigureTk(backend_tools.SaveFigureBase):
     def trigger(self, *args):
         NavigationToolbar2Tk.save_figure(
@@ -848,7 +927,6 @@ class HelpTk(backend_tools.ToolHelpBase):
 
 backend_tools.ToolSaveFigure = SaveFigureTk
 backend_tools.ToolConfigureSubplots = ConfigureSubplotsTk
-backend_tools.ToolSetCursor = SetCursorTk
 backend_tools.ToolRubberband = RubberbandTk
 backend_tools.ToolHelp = HelpTk
 backend_tools.ToolCopyToClipboard = backend_tools.ToolCopyToClipboardBase
@@ -867,13 +945,14 @@ class _BackendTk(_Backend):
         with _restore_foreground_window_at_end():
             if cbook._get_running_interactive_framework() is None:
                 cbook._setup_new_guiapp()
+                _c_internal_utils.Win32_SetProcessDpiAwareness_max()
             window = tk.Tk(className="matplotlib")
             window.withdraw()
 
             # Put a Matplotlib icon on the window rather than the default tk
             # icon.  Tkinter doesn't allow colour icons on linux systems, but
             # tk>=8.5 has a iconphoto command which we call directly.  See
-            # http://mail.python.org/pipermail/tkinter-discuss/2006-November/000954.html
+            # https://mail.python.org/pipermail/tkinter-discuss/2006-November/000954.html
             icon_fname = str(cbook._get_data_path(
                 'images/matplotlib_128.ppm'))
             icon_img = tk.PhotoImage(file=icon_fname, master=window)
