@@ -15,7 +15,6 @@ the Quiver code.
 """
 
 import math
-import weakref
 
 import numpy as np
 from numpy import ma
@@ -38,6 +37,11 @@ Call signature::
 *X*, *Y* define the arrow locations, *U*, *V* define the arrow directions, and
 *C* optionally sets the color.
 
+Each arrow is internally represented by a filled polygon with a default edge
+linewidth of 0. As a result, an arrow is rather a filled area, not a line with
+a head, and `.PolyCollection` properties like *linewidth*, *linestyle*,
+*facecolor*, etc. act accordingly.
+
 **Arrow size**
 
 The default settings auto-scales the length of the arrows to a reasonable size.
@@ -177,20 +181,29 @@ color : color or color sequence, optional
     Explicit color(s) for the arrows. If *C* has been set, *color* has no
     effect.
 
-    This is a synonym for the `~.PolyCollection` *facecolor* parameter.
+    This is a synonym for the `.PolyCollection` *facecolor* parameter.
 
 Other Parameters
 ----------------
+data : indexable object, optional
+    DATA_PARAMETER_PLACEHOLDER
+
 **kwargs : `~matplotlib.collections.PolyCollection` properties, optional
     All other keyword arguments are passed on to `.PolyCollection`:
 
-    %(PolyCollection_kwdoc)s
+    %(PolyCollection:kwdoc)s
+
+Returns
+-------
+`~matplotlib.quiver.Quiver`
 
 See Also
 --------
 .Axes.quiverkey : Add a key to a quiver plot.
 """ % docstring.interpd.params
 
+docstring.interpd.update(quiver_doc=_quiver_doc)
+
 
 class QuiverKey(martist.Artist):
     """Labelled arrow for use as a quiver plot scale key."""
@@ -259,21 +272,6 @@ class QuiverKey(martist.Artist):
         self.color = color
         self.label = label
         self._labelsep_inches = labelsep
-        self.labelsep = (self._labelsep_inches * Q.axes.figure.dpi)
-
-        # try to prevent closure over the real self
-        weak_self = weakref.ref(self)
-
-        def on_dpi_change(fig):
-            self_weakref = weak_self()
-            if self_weakref is not None:
-                self_weakref.labelsep = self_weakref._labelsep_inches * fig.dpi
-                # simple brute force update works because _init is called at
-                # the start of draw.
-                self_weakref._initialized = False
-
-        self._cid = Q.axes.figure.callbacks.connect(
-            'dpi_changed', on_dpi_change)
 
         self.labelpos = labelpos
         self.labelcolor = labelcolor
@@ -289,18 +287,16 @@ class QuiverKey(martist.Artist):
 
         if self.labelcolor is not None:
             self.text.set_color(self.labelcolor)
-        self._initialized = False
+        self._dpi_at_last_init = None
         self.zorder = Q.zorder + 0.1
 
-    def remove(self):
-        # docstring inherited
-        self.Q.axes.figure.callbacks.disconnect(self._cid)
-        self._cid = None
-        super().remove()  # pass the remove call up the stack
+    @property
+    def labelsep(self):
+        return self._labelsep_inches * self.Q.axes.figure.dpi
 
     def _init(self):
-        if True:  # not self._initialized:
-            if not self.Q._initialized:
+        if True:  # self._dpi_at_last_init != self.axes.figure.dpi
+            if self.Q._dpi_at_last_init != self.Q.axes.figure.dpi:
                 self.Q._init()
             self._set_transform()
             with cbook._setattr_cm(self.Q, pivot=self.pivot[self.labelpos],
@@ -323,7 +319,7 @@ class QuiverKey(martist.Artist):
                 self.vector.set_color(self.color)
             self.vector.set_transform(self.Q.get_transform())
             self.vector.set_figure(self.get_figure())
-            self._initialized = True
+            self._dpi_at_last_init = self.Q.axes.figure.dpi
 
     def _text_x(self, x):
         if self.labelpos == 'E':
@@ -494,30 +490,7 @@ class Quiver(mcollections.PolyCollection):
                          closed=False, **kw)
         self.polykw = kw
         self.set_UVC(U, V, C)
-        self._initialized = False
-
-        weak_self = weakref.ref(self)  # Prevent closure over the real self.
-
-        def on_dpi_change(fig):
-            self_weakref = weak_self()
-            if self_weakref is not None:
-                # vertices depend on width, span which in turn depend on dpi
-                self_weakref._new_UV = True
-                # simple brute force update works because _init is called at
-                # the start of draw.
-                self_weakref._initialized = False
-
-        self._cid = ax.figure.callbacks.connect('dpi_changed', on_dpi_change)
-
-    @_api.deprecated("3.3", alternative="axes")
-    def ax(self):
-        return self.axes
-
-    def remove(self):
-        # docstring inherited
-        self.axes.figure.callbacks.disconnect(self._cid)
-        self._cid = None
-        super().remove()  # pass the remove call up the stack
+        self._dpi_at_last_init = None
 
     def _init(self):
         """
@@ -526,7 +499,7 @@ class Quiver(mcollections.PolyCollection):
         """
         # It seems that there are not enough event notifications
         # available to have this work on an as-needed basis at present.
-        if True:  # not self._initialized:
+        if True:  # self._dpi_at_last_init != self.axes.figure.dpi
             trans = self._set_transform()
             self.span = trans.inverted().transform_bbox(self.axes.bbox).width
             if self.width is None:
@@ -534,10 +507,11 @@ class Quiver(mcollections.PolyCollection):
                 self.width = 0.06 * self.span / sn
 
             # _make_verts sets self.scale if not already specified
-            if not self._initialized and self.scale is None:
+            if (self._dpi_at_last_init != self.axes.figure.dpi
+                    and self.scale is None):
                 self._make_verts(self.U, self.V, self.angles)
 
-            self._initialized = True
+            self._dpi_at_last_init = self.axes.figure.dpi
 
     def get_datalim(self, transData):
         trans = self.get_transform()
@@ -553,7 +527,6 @@ class Quiver(mcollections.PolyCollection):
         self._init()
         verts = self._make_verts(self.U, self.V, self.angles)
         self.set_verts(verts, closed=False)
-        self._new_UV = False
         super().draw(renderer)
         self.stale = False
 
@@ -582,7 +555,6 @@ class Quiver(mcollections.PolyCollection):
         self.Umask = mask
         if C is not None:
             self.set_array(C)
-        self._new_UV = True
         self.stale = True
 
     def _dots_per_unit(self, units):
@@ -884,11 +856,14 @@ barbs : `~matplotlib.quiver.Barbs`
 
 Other Parameters
 ----------------
+data : indexable object, optional
+    DATA_PARAMETER_PLACEHOLDER
+
 **kwargs
     The barbs can further be customized using `.PolyCollection` keyword
     arguments:
 
-    %(PolyCollection_kwdoc)s
+    %(PolyCollection:kwdoc)s
 """ % docstring.interpd.params
 
 docstring.interpd.update(barbs_doc=_barbs_doc)
@@ -1147,8 +1122,10 @@ class Barbs(mcollections.PolyCollection):
         return barb_list
 
     def set_UVC(self, U, V, C=None):
-        self.u = ma.masked_invalid(U, copy=False).ravel()
-        self.v = ma.masked_invalid(V, copy=False).ravel()
+        # We need to ensure we have a copy, not a reference to an array that
+        # might change before draw().
+        self.u = ma.masked_invalid(U, copy=True).ravel()
+        self.v = ma.masked_invalid(V, copy=True).ravel()
 
         # Flip needs to have the same number of entries as everything else.
         # Use broadcast_to to avoid a bloated array of identical values.
@@ -1159,7 +1136,7 @@ class Barbs(mcollections.PolyCollection):
             flip = self.flip
 
         if C is not None:
-            c = ma.masked_invalid(C, copy=False).ravel()
+            c = ma.masked_invalid(C, copy=True).ravel()
             x, y, u, v, c, flip = cbook.delete_masked_points(
                 self.x.ravel(), self.y.ravel(), self.u, self.v, c,
                 flip.ravel())
