@@ -2,14 +2,14 @@
 Classes for including text in a figure.
 """
 
-import contextlib
 import logging
 import math
 import weakref
 
 import numpy as np
 
-from . import artist, cbook, docstring, rcParams
+import matplotlib as mpl
+from . import _api, artist, cbook, docstring
 from .artist import Artist
 from .font_manager import FontProperties
 from .patches import FancyArrowPatch, FancyBboxPatch, Rectangle
@@ -21,20 +21,6 @@ from .transforms import (
 _log = logging.getLogger(__name__)
 
 
-@contextlib.contextmanager
-def _wrap_text(textobj):
-    """Temporarily inserts newlines if the wrap option is enabled."""
-    if textobj.get_wrap():
-        old_text = textobj.get_text()
-        try:
-            textobj.set_text(textobj._get_wrapped_text())
-            yield textobj
-        finally:
-            textobj.set_text(old_text)
-    else:
-        yield textobj
-
-
 # Extracted from Text's method to serve as a function
 def get_rotation(rotation):
     """
@@ -65,12 +51,13 @@ def get_rotation(rotation):
 
 def _get_textbox(text, renderer):
     """
-    Calculate the bounding box of the text. Unlike
-    :meth:`matplotlib.text.Text.get_extents` method, The bbox size of
-    the text before the rotation is calculated.
+    Calculate the bounding box of the text.
+
+    The bbox position takes text rotation into account, but the width and
+    height are those of the unrotated box (unlike `.Text.get_window_extent`).
     """
     # TODO : This function may move into the Text class as a method. As a
-    # matter of fact, The information from the _get_textbox function
+    # matter of fact, the information from the _get_textbox function
     # should be available during the Text._get_layout() call, which is
     # called within the _get_textbox. So, it would better to move this
     # function as a method with some refactoring of _get_layout method.
@@ -101,6 +88,7 @@ def _get_textbox(text, renderer):
     return x_box, y_box, w_box, h_box
 
 
+@docstring.interpd
 @cbook._define_aliases({
     "color": ["c"],
     "fontfamily": ["family"],
@@ -136,6 +124,9 @@ class Text(Artist):
                  rotation_mode=None,
                  usetex=None,          # defaults to rcParams['text.usetex']
                  wrap=False,
+                 transform_rotates_text=False,
+                 *,
+                 parse_math=True,
                  **kwargs
                  ):
         """
@@ -143,20 +134,23 @@ class Text(Artist):
 
         Valid keyword arguments are:
 
-        %(Text)s
+        %(Text:kwdoc)s
         """
-        Artist.__init__(self)
+        super().__init__()
         self._x, self._y = x, y
         self._text = ''
         self.set_text(text)
-        self.set_color(color if color is not None else rcParams["text.color"])
+        self.set_color(
+            color if color is not None else mpl.rcParams["text.color"])
         self.set_fontproperties(fontproperties)
         self.set_usetex(usetex)
+        self.set_parse_math(parse_math)
         self.set_wrap(wrap)
         self.set_verticalalignment(verticalalignment)
         self.set_horizontalalignment(horizontalalignment)
         self._multialignment = multialignment
         self._rotation = rotation
+        self._transform_rotates_text = transform_rotates_text
         self._bbox_patch = None  # a FancyBboxPatch instance
         self._renderer = None
         if linespacing is None:
@@ -167,8 +161,7 @@ class Text(Artist):
 
     def update(self, kwargs):
         # docstring inherited
-        # make a copy so we do not mutate user input!
-        kwargs = dict(kwargs)
+        kwargs = cbook.normalize_kwargs(kwargs, Text)
         sentinel = object()  # bbox can be None, so use another sentinel.
         # Update fontproperties first, as it has lowest priority.
         fontproperties = kwargs.pop("fontproperties", sentinel)
@@ -230,7 +223,20 @@ class Text(Artist):
 
     def get_rotation(self):
         """Return the text angle in degrees between 0 and 360."""
-        return get_rotation(self._rotation)  # string_or_number -> number
+        if self.get_transform_rotates_text():
+            angle = get_rotation(self._rotation)
+            x, y = self.get_unitless_position()
+            angles = [angle, ]
+            pts = [[x, y]]
+            return self.get_transform().transform_angles(angles, pts).item(0)
+        else:
+            return get_rotation(self._rotation)  # string_or_number -> number
+
+    def get_transform_rotates_text(self):
+        """
+        Return whether rotations of the transform affect the text direction.
+        """
+        return self._transform_rotates_text
 
     def set_rotation_mode(self, m):
         """
@@ -243,7 +249,7 @@ class Text(Artist):
             aligned according to their horizontal and vertical alignments.  If
             ``"anchor"``, then alignment occurs before rotation.
         """
-        cbook._check_in_list(["anchor", "default", None], rotation_mode=m)
+        _api.check_in_list(["anchor", "default", None], rotation_mode=m)
         self._rotation_mode = m
         self.stale = True
 
@@ -253,7 +259,7 @@ class Text(Artist):
 
     def update_from(self, other):
         # docstring inherited
-        Artist.update_from(self, other)
+        super().update_from(other)
         self._color = other._color
         self._multialignment = other._multialignment
         self._verticalalignment = other._verticalalignment
@@ -261,17 +267,33 @@ class Text(Artist):
         self._fontproperties = other._fontproperties.copy()
         self._usetex = other._usetex
         self._rotation = other._rotation
+        self._transform_rotates_text = other._transform_rotates_text
         self._picker = other._picker
         self._linespacing = other._linespacing
         self.stale = True
 
+    def _get_layout_cache_key(self, renderer=None):
+        """
+        Return a hashable tuple of properties that lets `_get_layout` know
+        whether a previously computed layout can be reused.
+        """
+        x, y = self.get_unitless_position()
+        renderer = renderer or self._renderer
+        return (
+            x, y, self.get_text(), hash(self._fontproperties),
+            self._verticalalignment, self._horizontalalignment,
+            self._linespacing,
+            self._rotation, self._rotation_mode, self._transform_rotates_text,
+            self.figure.dpi, weakref.ref(renderer),
+        )
+
     def _get_layout(self, renderer):
         """
         Return the extent (bbox) of the text together with
         multiple-alignment information. Note that it returns an extent
         of a rotated text when necessary.
         """
-        key = self.get_prop_tup(renderer=renderer)
+        key = self._get_layout_cache_key(renderer=renderer)
         if key in self._cached:
             return self._cached[key]
 
@@ -451,20 +473,12 @@ class Text(Artist):
             else:
                 if pad is None:
                     pad = 0.3
-
             # boxstyle could be a callable or a string
             if isinstance(boxstyle, str) and "pad" not in boxstyle:
                 boxstyle += ",pad=%0.2f" % pad
-
-            bbox_transmuter = props.pop("bbox_transmuter", None)
-
             self._bbox_patch = FancyBboxPatch(
-                                    (0., 0.),
-                                    1., 1.,
-                                    boxstyle=boxstyle,
-                                    bbox_transmuter=bbox_transmuter,
-                                    transform=IdentityTransform(),
-                                    **props)
+                (0, 0), 1, 1,
+                boxstyle=boxstyle, transform=IdentityTransform(), **props)
         else:
             self._bbox_patch = None
 
@@ -484,17 +498,12 @@ class Text(Artist):
         This method should be used when the position and size of the bbox needs
         to be updated before actually drawing the bbox.
         """
-
         if self._bbox_patch:
-
-            trans = self.get_transform()
-
             # don't use self.get_unitless_position here, which refers to text
             # position in Text:
             posx = float(self.convert_xunits(self._x))
             posy = float(self.convert_yunits(self._y))
-
-            posx, posy = trans.transform((posx, posy))
+            posx, posy = self.get_transform().transform((posx, posy))
 
             x_box, y_box, w_box, h_box = _get_textbox(self, renderer)
             self._bbox_patch.set_bounds(0., 0., w_box, h_box)
@@ -505,22 +514,6 @@ class Text(Artist):
             fontsize_in_pixel = renderer.points_to_pixels(self.get_size())
             self._bbox_patch.set_mutation_scale(fontsize_in_pixel)
 
-    def _draw_bbox(self, renderer, posx, posy):
-        """
-        Update the location and size of the bbox (`.patches.FancyBboxPatch`),
-        and draw.
-        """
-
-        x_box, y_box, w_box, h_box = _get_textbox(self, renderer)
-        self._bbox_patch.set_bounds(0., 0., w_box, h_box)
-        theta = np.deg2rad(self.get_rotation())
-        tr = Affine2D().rotate(theta)
-        tr = tr.translate(posx + x_box, posy + y_box)
-        self._bbox_patch.set_transform(tr)
-        fontsize_in_pixel = renderer.points_to_pixels(self.get_size())
-        self._bbox_patch.set_mutation_scale(fontsize_in_pixel)
-        self._bbox_patch.draw(renderer)
-
     def _update_clip_properties(self):
         clipprops = dict(clip_box=self.clipbox,
                          clip_path=self._clippath,
@@ -554,6 +547,14 @@ class Text(Artist):
         Parameters
         ----------
         wrap : bool
+
+        Notes
+        -----
+        Wrapping does not work together with
+        ``savefig(..., bbox_inches='tight')`` (which is also used internally
+        by ``%matplotlib inline`` in IPython/Jupyter). The 'tight' setting
+        rescales the canvas to accommodate all content and happens before
+        wrapping.
         """
         self._wrap = wrap
 
@@ -618,9 +619,12 @@ class Text(Artist):
 
     def _get_wrapped_text(self):
         """
-        Return a copy of the text with new lines added, so that
-        the text is wrapped relative to the parent figure.
+        Return a copy of the text string with new lines added so that the text
+        is wrapped relative to the parent figure (if `get_wrap` is True).
         """
+        if not self.get_wrap():
+            return self.get_text()
+
         # Not fit to handle breaking up latex syntax correctly, so
         # ignore latex for now.
         if self.get_usetex():
@@ -677,55 +681,57 @@ class Text(Artist):
 
         renderer.open_group('text', self.get_gid())
 
-        with _wrap_text(self) as textobj:
-            bbox, info, descent = textobj._get_layout(renderer)
-            trans = textobj.get_transform()
+        with self._cm_set(text=self._get_wrapped_text()):
+            bbox, info, descent = self._get_layout(renderer)
+            trans = self.get_transform()
 
-            # don't use textobj.get_position here, which refers to text
+            # don't use self.get_position here, which refers to text
             # position in Text:
-            posx = float(textobj.convert_xunits(textobj._x))
-            posy = float(textobj.convert_yunits(textobj._y))
+            posx = float(self.convert_xunits(self._x))
+            posy = float(self.convert_yunits(self._y))
             posx, posy = trans.transform((posx, posy))
             if not np.isfinite(posx) or not np.isfinite(posy):
                 _log.warning("posx and posy should be finite values")
                 return
             canvasw, canvash = renderer.get_canvas_width_height()
 
-            # draw the FancyBboxPatch
-            if textobj._bbox_patch:
-                textobj._draw_bbox(renderer, posx, posy)
+            # Update the location and size of the bbox
+            # (`.patches.FancyBboxPatch`), and draw it.
+            if self._bbox_patch:
+                self.update_bbox_position_size(renderer)
+                self._bbox_patch.draw(renderer)
 
             gc = renderer.new_gc()
-            gc.set_foreground(textobj.get_color())
-            gc.set_alpha(textobj.get_alpha())
-            gc.set_url(textobj._url)
-            textobj._set_gc_clip(gc)
+            gc.set_foreground(self.get_color())
+            gc.set_alpha(self.get_alpha())
+            gc.set_url(self._url)
+            self._set_gc_clip(gc)
 
-            angle = textobj.get_rotation()
+            angle = self.get_rotation()
 
             for line, wh, x, y in info:
 
-                mtext = textobj if len(info) == 1 else None
+                mtext = self if len(info) == 1 else None
                 x = x + posx
                 y = y + posy
                 if renderer.flipy():
                     y = canvash - y
-                clean_line, ismath = textobj._preprocess_math(line)
+                clean_line, ismath = self._preprocess_math(line)
 
-                if textobj.get_path_effects():
+                if self.get_path_effects():
                     from matplotlib.patheffects import PathEffectRenderer
                     textrenderer = PathEffectRenderer(
-                                        textobj.get_path_effects(), renderer)
+                        self.get_path_effects(), renderer)
                 else:
                     textrenderer = renderer
 
-                if textobj.get_usetex():
+                if self.get_usetex():
                     textrenderer.draw_tex(gc, x, y, clean_line,
-                                          textobj._fontproperties, angle,
+                                          self._fontproperties, angle,
                                           mtext=mtext)
                 else:
                     textrenderer.draw_text(gc, x, y, clean_line,
-                                           textobj._fontproperties, angle,
+                                           self._fontproperties, angle,
                                            ismath=ismath, mtext=mtext)
 
         gc.restore()
@@ -831,6 +837,8 @@ class Text(Artist):
         # specified with 'set_x' and 'set_y'.
         return self._x, self._y
 
+    # When removing, also remove the hash(color) check in set_color()
+    @_api.deprecated("3.5")
     def get_prop_tup(self, renderer=None):
         """
         Return a hashable tuple of properties.
@@ -845,6 +853,7 @@ class Text(Artist):
                 self._verticalalignment, self._horizontalalignment,
                 hash(self._fontproperties),
                 self._rotation, self._rotation_mode,
+                self._transform_rotates_text,
                 self.figure.dpi, weakref.ref(renderer),
                 self._linespacing
                 )
@@ -856,7 +865,7 @@ class Text(Artist):
     def get_verticalalignment(self):
         """
         Return the vertical alignment as a string.  Will be one of
-        'top', 'center', 'bottom' or 'baseline'.
+        'top', 'center', 'bottom', 'baseline' or 'center_baseline'.
         """
         return self._verticalalignment
 
@@ -881,7 +890,6 @@ class Text(Artist):
             ``self.figure.dpi`` (*not* the renderer dpi); should be set e.g. if
             to match regions with a figure saved with a custom dpi value.
         """
-        #return _unit_box
         if not self.get_visible():
             return Bbox.unit()
         if dpi is None:
@@ -933,7 +941,12 @@ class Text(Artist):
         ----------
         color : color
         """
-        # Make sure it is hashable, or get_prop_tup will fail.
+        # "auto" is only supported by axisartist, but we can just let it error
+        # out at draw time for simplicity.
+        if not cbook._str_equal(color, "auto"):
+            mpl.colors._check_color_like(color=color)
+        # Make sure it is hashable, or get_prop_tup will fail (remove this once
+        # get_prop_tup is removed).
         try:
             hash(color)
         except TypeError:
@@ -949,7 +962,7 @@ class Text(Artist):
         ----------
         align : {'center', 'right', 'left'}
         """
-        cbook._check_in_list(['center', 'right', 'left'], align=align)
+        _api.check_in_list(['center', 'right', 'left'], align=align)
         self._horizontalalignment = align
         self.stale = True
 
@@ -965,7 +978,7 @@ class Text(Artist):
         ----------
         align : {'left', 'right', 'center'}
         """
-        cbook._check_in_list(['center', 'right', 'left'], align=align)
+        _api.check_in_list(['center', 'right', 'left'], align=align)
         self._multialignment = align
         self.stale = True
 
@@ -1054,6 +1067,40 @@ class Text(Artist):
         self._fontproperties.set_size(fontsize)
         self.stale = True
 
+    def get_math_fontfamily(self):
+        """
+        Return the font family name for math text rendered by Matplotlib.
+
+        The default value is :rc:`mathtext.fontset`.
+
+        See Also
+        --------
+        set_math_fontfamily
+        """
+        return self._fontproperties.get_math_fontfamily()
+
+    def set_math_fontfamily(self, fontfamily):
+        """
+        Set the font family for math text rendered by Matplotlib.
+
+        This does only affect Matplotlib's own math renderer. It has no effect
+        when rendering with TeX (``usetex=True``).
+
+        Parameters
+        ----------
+        fontfamily : str
+            The name of the font family.
+
+            Available font families are defined in the
+            :ref:`matplotlibrc.template file
+            <customizing-with-matplotlibrc-files>`.
+
+        See Also
+        --------
+        get_math_fontfamily
+        """
+        self._fontproperties.set_math_fontfamily(fontfamily)
+
     def set_fontweight(self, weight):
         """
         Set the font weight.
@@ -1134,6 +1181,17 @@ class Text(Artist):
         self._rotation = s
         self.stale = True
 
+    def set_transform_rotates_text(self, t):
+        """
+        Whether rotations of the transform affect the text direction.
+
+        Parameters
+        ----------
+        t : bool
+        """
+        self._transform_rotates_text = t
+        self.stale = True
+
     def set_verticalalignment(self, align):
         """
         Set the vertical alignment.
@@ -1142,7 +1200,7 @@ class Text(Artist):
         ----------
         align : {'center', 'top', 'bottom', 'baseline', 'center_baseline'}
         """
-        cbook._check_in_list(
+        _api.check_in_list(
             ['top', 'bottom', 'center', 'baseline', 'center_baseline'],
             align=align)
         self._verticalalignment = align
@@ -1174,7 +1232,8 @@ class Text(Artist):
         - If *self* is configured to use TeX, return *s* unchanged except that
           a single space gets escaped, and the flag "TeX".
         - Otherwise, if *s* is mathtext (has an even number of unescaped dollar
-          signs), return *s* and the flag True.
+          signs) and ``parse_math`` is not set to False, return *s* and the
+          flag True.
         - Otherwise, return *s* with dollar signs unescaped, and the flag
           False.
         """
@@ -1182,6 +1241,8 @@ class Text(Artist):
             if s == " ":
                 s = r"\ "
             return s, "TeX"
+        elif not self.get_parse_math():
+            return s, False
         elif cbook.is_math_text(s):
             return s, True
         else:
@@ -1210,7 +1271,7 @@ class Text(Artist):
             :rc:`text.usetex`.
         """
         if usetex is None:
-            self._usetex = rcParams['text.usetex']
+            self._usetex = mpl.rcParams['text.usetex']
         else:
             self._usetex = bool(usetex)
         self.stale = True
@@ -1219,6 +1280,22 @@ class Text(Artist):
         """Return whether this `Text` object uses TeX for rendering."""
         return self._usetex
 
+    def set_parse_math(self, parse_math):
+        """
+        Override switch to disable any mathtext parsing for this `Text`.
+
+        Parameters
+        ----------
+        parse_math : bool
+            If False, this `Text` will never use mathtext.  If True, mathtext
+            will be used if there is an even number of unescaped dollar signs.
+        """
+        self._parse_math = bool(parse_math)
+
+    def get_parse_math(self):
+        """Return whether mathtext parsing is considered for this `Text`."""
+        return self._parse_math
+
     def set_fontname(self, fontname):
         """
         Alias for `set_family`.
@@ -1238,10 +1315,6 @@ class Text(Artist):
         return self.set_family(fontname)
 
 
-docstring.interpd.update(Text=artist.kwdoc(Text))
-docstring.dedent_interpd(Text.__init__)
-
-
 class OffsetFrom:
     """Callable helper class for working with `Annotation`."""
 
@@ -1275,7 +1348,7 @@ class OffsetFrom:
         ----------
         unit : {'points', 'pixels'}
         """
-        cbook._check_in_list(["points", "pixels"], unit=unit)
+        _api.check_in_list(["points", "pixels"], unit=unit)
         self._unit = unit
 
     def get_unit(self):
@@ -1392,6 +1465,8 @@ class _AnnotationBase:
         bbox_name, unit = s_
         # if unit is offset-like
         if bbox_name == "figure":
+            bbox0 = self.figure.figbbox
+        elif bbox_name == "subfigure":
             bbox0 = self.figure.bbox
         elif bbox_name == "axes":
             bbox0 = self.axes.bbox
@@ -1409,7 +1484,6 @@ class _AnnotationBase:
         if xy0 is not None:
             # reference x, y in display coordinate
             ref_x, ref_y = xy0
-            from matplotlib.transforms import Affine2D
             if unit == "points":
                 # dots per points
                 dpp = self.figure.get_dpi() / 72.
@@ -1557,8 +1631,7 @@ class Annotation(Text, _AnnotationBase):
         Parameters
         ----------
         text : str
-            The text of the annotation.  *s* is a deprecated synonym for this
-            parameter.
+            The text of the annotation.
 
         xy : (float, float)
             The point *(x, y)* to annotate. The coordinate system is determined
@@ -1576,19 +1649,27 @@ class Annotation(Text, _AnnotationBase):
 
             - One of the following strings:
 
-              =================   =============================================
-              Value               Description
-              =================   =============================================
-              'figure points'     Points from the lower left of the figure
-              'figure pixels'     Pixels from the lower left of the figure
-              'figure fraction'   Fraction of figure from lower left
-              'axes points'       Points from lower left corner of axes
-              'axes pixels'       Pixels from lower left corner of axes
-              'axes fraction'     Fraction of axes from lower left
-              'data'              Use the coordinate system of the object being
-                                  annotated (default)
-              'polar'             *(theta, r)* if not native 'data' coordinates
-              =================   =============================================
+              ==================== ============================================
+              Value                Description
+              ==================== ============================================
+              'figure points'      Points from the lower left of the figure
+              'figure pixels'      Pixels from the lower left of the figure
+              'figure fraction'    Fraction of figure from lower left
+              'subfigure points'   Points from the lower left of the subfigure
+              'subfigure pixels'   Pixels from the lower left of the subfigure
+              'subfigure fraction' Fraction of subfigure from lower left
+              'axes points'        Points from lower left corner of axes
+              'axes pixels'        Pixels from lower left corner of axes
+              'axes fraction'      Fraction of axes from lower left
+              'data'               Use the coordinate system of the object
+                                   being annotated (default)
+              'polar'              *(theta, r)* if not native 'data'
+                                   coordinates
+              ==================== ============================================
+
+              Note that 'subfigure pixels' and 'figure pixels' are the same
+              for the parent figure, so users who want code that is usable in
+              a subfigure can use 'subfigure pixels'.
 
             - An `.Artist`: *xy* is interpreted as a fraction of the artist's
               `~matplotlib.transforms.Bbox`. E.g. *(0, 0)* would be the lower
@@ -1629,7 +1710,13 @@ class Annotation(Text, _AnnotationBase):
 
         arrowprops : dict, optional
             The properties used to draw a `.FancyArrowPatch` arrow between the
-            positions *xy* and *xytext*.
+            positions *xy* and *xytext*.  Defaults to None, i.e. no arrow is
+            drawn.
+
+            For historical reasons there are two different ways to specify
+            arrows, "simple" and "fancy":
+
+            **Simple arrow:**
 
             If *arrowprops* does not contain the key 'arrowstyle' the
             allowed keys are:
@@ -1644,35 +1731,22 @@ class Annotation(Text, _AnnotationBase):
             ?            Any key to :class:`matplotlib.patches.FancyArrowPatch`
             ==========   ======================================================
 
-            If *arrowprops* contains the key 'arrowstyle' the
-            above keys are forbidden.  The allowed values of
-            ``'arrowstyle'`` are:
+            The arrow is attached to the edge of the text box, the exact
+            position (corners or centers) depending on where it's pointing to.
 
-            ============   =============================================
-            Name           Attrs
-            ============   =============================================
-            ``'-'``        None
-            ``'->'``       head_length=0.4,head_width=0.2
-            ``'-['``       widthB=1.0,lengthB=0.2,angleB=None
-            ``'|-|'``      widthA=1.0,widthB=1.0
-            ``'-|>'``      head_length=0.4,head_width=0.2
-            ``'<-'``       head_length=0.4,head_width=0.2
-            ``'<->'``      head_length=0.4,head_width=0.2
-            ``'<|-'``      head_length=0.4,head_width=0.2
-            ``'<|-|>'``    head_length=0.4,head_width=0.2
-            ``'fancy'``    head_length=0.4,head_width=0.4,tail_width=0.4
-            ``'simple'``   head_length=0.5,head_width=0.5,tail_width=0.2
-            ``'wedge'``    tail_width=0.3,shrink_factor=0.5
-            ============   =============================================
+            **Fancy arrow:**
 
-            Valid keys for `~matplotlib.patches.FancyArrowPatch` are:
+            This is used if 'arrowstyle' is provided in the *arrowprops*.
+
+            Valid keys are the following `~matplotlib.patches.FancyArrowPatch`
+            parameters:
 
             ===============  ==================================================
             Key              Description
             ===============  ==================================================
             arrowstyle       the arrow style
             connectionstyle  the connection style
-            relpos           default is (0.5, 0.5)
+            relpos           see below; default is (0.5, 0.5)
             patchA           default is bounding box of the text
             patchB           default is None
             shrinkA          default is 2 points
@@ -1682,7 +1756,12 @@ class Annotation(Text, _AnnotationBase):
             ?                any key for :class:`matplotlib.patches.PathPatch`
             ===============  ==================================================
 
-            Defaults to None, i.e. no arrow is drawn.
+            The exact starting point position of the arrow is defined by
+            *relpos*. It's a tuple of relative coordinates of the text box,
+            where (0, 0) is the lower left corner and (1, 1) is the upper
+            right corner. Values <0 and >1 are supported and specify points
+            outside the text box. By default (0.5, 0.5) the starting point is
+            centered in the text box.
 
         annotation_clip : bool or None, default: None
             Whether to draw the annotation when the annotation point *xy* is
@@ -1714,9 +1793,9 @@ class Annotation(Text, _AnnotationBase):
         if (xytext is None and
                 textcoords is not None and
                 textcoords != xycoords):
-            cbook._warn_external("You have used the `textcoords` kwarg, but "
-                                 "not the `xytext` kwarg.  This can lead to "
-                                 "surprising results.")
+            _api.warn_external("You have used the `textcoords` kwarg, but "
+                               "not the `xytext` kwarg.  This can lead to "
+                               "surprising results.")
 
         # clean up textcoords and assign default
         if textcoords is None:
@@ -1833,7 +1912,7 @@ class Annotation(Text, _AnnotationBase):
             # Ignore frac--it is useless.
             frac = d.pop('frac', None)
             if frac is not None:
-                cbook._warn_external(
+                _api.warn_external(
                     "'frac' option in 'arrowprops' is no longer supported;"
                     " use 'headlength' to set the head length in points.")
             headlength = d.pop('headlength', 12)
@@ -1889,6 +1968,8 @@ class Annotation(Text, _AnnotationBase):
             self._renderer = renderer
         if not self.get_visible() or not self._check_xy(renderer):
             return
+        # Update text positions before `Text.draw` would, so that the
+        # FancyArrowPatch is correctly positioned.
         self.update_positions(renderer)
         self.update_bbox_position_size(renderer)
         if self.arrow_patch is not None:   # FancyArrowPatch
@@ -1900,21 +1981,10 @@ class Annotation(Text, _AnnotationBase):
         Text.draw(self, renderer)
 
     def get_window_extent(self, renderer=None):
-        """
-        Return the `.Bbox` bounding the text and arrow, in display units.
-
-        Parameters
-        ----------
-        renderer : Renderer, optional
-            A renderer is needed to compute the bounding box.  If the artist
-            has already been drawn, the renderer is cached; thus, it is only
-            necessary to pass this argument when calling `get_window_extent`
-            before the first `draw`.  In practice, it is usually easier to
-            trigger a draw first (e.g. by saving the figure).
-        """
+        # docstring inherited
         # This block is the same as in Text.get_window_extent, but we need to
         # set the renderer before calling update_positions().
-        if not self.get_visible():
+        if not self.get_visible() or not self._check_xy(renderer):
             return Bbox.unit()
         if renderer is not None:
             self._renderer = renderer
@@ -1933,5 +2003,11 @@ class Annotation(Text, _AnnotationBase):
 
         return Bbox.union(bboxes)
 
+    def get_tightbbox(self, renderer):
+        # docstring inherited
+        if not self._check_xy(renderer):
+            return Bbox.null()
+        return super().get_tightbbox(renderer)
+
 
 docstring.interpd.update(Annotation=Annotation.__init__.__doc__)
