@@ -1,11 +1,8 @@
 """
 A collection of functions and objects for creating or placing inset axes.
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-from matplotlib import docstring
-import six
+from matplotlib import _api, docstring
 from matplotlib.offsetbox import AnchoredOffsetbox
 from matplotlib.patches import Patch, Rectangle
 from matplotlib.path import Path
@@ -16,7 +13,7 @@ from . import axes_size as Size
 from .parasite_axes import HostAxes
 
 
-class InsetPosition(object):
+class InsetPosition:
     @docstring.dedent_interpd
     def __init__(self, parent, lbwh):
         """
@@ -63,7 +60,7 @@ class InsetPosition(object):
 class AnchoredLocatorBase(AnchoredOffsetbox):
     def __init__(self, bbox_to_anchor, offsetbox, loc,
                  borderpad=0.5, bbox_transform=None):
-        super(AnchoredLocatorBase, self).__init__(
+        super().__init__(
             loc, pad=0., child=None, borderpad=borderpad,
             bbox_to_anchor=bbox_to_anchor, bbox_transform=bbox_transform
         )
@@ -90,8 +87,7 @@ class AnchoredLocatorBase(AnchoredOffsetbox):
 class AnchoredSizeLocator(AnchoredLocatorBase):
     def __init__(self, bbox_to_anchor, x_size, y_size, loc,
                  borderpad=0.5, bbox_transform=None):
-
-        super(AnchoredSizeLocator, self).__init__(
+        super().__init__(
             bbox_to_anchor, None, loc,
             borderpad=borderpad, bbox_transform=bbox_transform
         )
@@ -100,21 +96,20 @@ class AnchoredSizeLocator(AnchoredLocatorBase):
         self.y_size = Size.from_any(y_size)
 
     def get_extent(self, renderer):
-        x, y, w, h = self.get_bbox_to_anchor().bounds
-
+        bbox = self.get_bbox_to_anchor()
         dpi = renderer.points_to_pixels(72.)
 
         r, a = self.x_size.get_size(renderer)
-        width = w*r + a*dpi
-
+        width = bbox.width * r + a * dpi
         r, a = self.y_size.get_size(renderer)
-        height = h*r + a*dpi
+        height = bbox.height * r + a * dpi
+
         xd, yd = 0, 0
 
         fontsize = renderer.points_to_pixels(self.prop.get_size_in_points())
         pad = self.pad * fontsize
 
-        return width+2*pad, height+2*pad, xd+pad, yd+pad
+        return width + 2 * pad, height + 2 * pad, xd + pad, yd + pad
 
 
 class AnchoredZoomLocator(AnchoredLocatorBase):
@@ -122,26 +117,21 @@ class AnchoredZoomLocator(AnchoredLocatorBase):
                  borderpad=0.5,
                  bbox_to_anchor=None,
                  bbox_transform=None):
-
         self.parent_axes = parent_axes
         self.zoom = zoom
-
         if bbox_to_anchor is None:
             bbox_to_anchor = parent_axes.bbox
-
-        super(AnchoredZoomLocator, self).__init__(
+        super().__init__(
             bbox_to_anchor, None, loc, borderpad=borderpad,
             bbox_transform=bbox_transform)
 
     def get_extent(self, renderer):
-        bb = TransformedBbox(self.axes.viewLim,
-                             self.parent_axes.transData)
-
-        x, y, w, h = bb.bounds
+        bb = TransformedBbox(self.axes.viewLim, self.parent_axes.transData)
         fontsize = renderer.points_to_pixels(self.prop.get_size_in_points())
         pad = self.pad * fontsize
-
-        return abs(w*self.zoom)+2*pad, abs(h*self.zoom)+2*pad, pad, pad
+        return (abs(bb.width * self.zoom) + 2 * pad,
+                abs(bb.height * self.zoom) + 2 * pad,
+                pad, pad)
 
 
 class BboxPatch(Patch):
@@ -157,34 +147,21 @@ class BboxPatch(Patch):
 
         **kwargs
             Patch properties. Valid arguments include:
-            %(Patch)s
+
+            %(Patch:kwdoc)s
         """
         if "transform" in kwargs:
             raise ValueError("transform should not be set")
 
         kwargs["transform"] = IdentityTransform()
-        Patch.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self.bbox = bbox
 
     def get_path(self):
+        # docstring inherited
         x0, y0, x1, y1 = self.bbox.extents
-
-        verts = [(x0, y0),
-                 (x1, y0),
-                 (x1, y1),
-                 (x0, y1),
-                 (x0, y0),
-                 (0, 0)]
-
-        codes = [Path.MOVETO,
-                 Path.LINETO,
-                 Path.LINETO,
-                 Path.LINETO,
-                 Path.LINETO,
-                 Path.CLOSEPOLY]
-
-        return Path(verts, codes)
-    get_path.__doc__ = Patch.get_path.__doc__
+        return Path([(x0, y0), (x1, y0), (x1, y1), (x0, y1), (x0, y0)],
+                    closed=True)
 
 
 class BboxConnector(Patch):
@@ -254,25 +231,14 @@ class BboxConnector(Patch):
             corner of *bbox2*.
         """
         if isinstance(bbox1, Rectangle):
-            transform = bbox1.get_transfrom()
-            bbox1 = Bbox.from_bounds(0, 0, 1, 1)
-            bbox1 = TransformedBbox(bbox1, transform)
-
+            bbox1 = TransformedBbox(Bbox.unit(), bbox1.get_transform())
         if isinstance(bbox2, Rectangle):
-            transform = bbox2.get_transform()
-            bbox2 = Bbox.from_bounds(0, 0, 1, 1)
-            bbox2 = TransformedBbox(bbox2, transform)
-
+            bbox2 = TransformedBbox(Bbox.unit(), bbox2.get_transform())
         if loc2 is None:
             loc2 = loc1
-
         x1, y1 = BboxConnector.get_bbox_edge_pos(bbox1, loc1)
         x2, y2 = BboxConnector.get_bbox_edge_pos(bbox2, loc2)
-
-        verts = [[x1, y1], [x2, y2]]
-        codes = [Path.MOVETO, Path.LINETO]
-
-        return Path(verts, codes)
+        return Path([[x1, y1], [x2, y2]])
 
     @docstring.dedent_interpd
     def __init__(self, bbox1, bbox2, loc1, loc2=None, **kwargs):
@@ -303,22 +269,27 @@ class BboxConnector(Patch):
 
         **kwargs
             Patch properties for the line drawn. Valid arguments include:
-            %(Patch)s
+
+            %(Patch:kwdoc)s
         """
         if "transform" in kwargs:
             raise ValueError("transform should not be set")
 
         kwargs["transform"] = IdentityTransform()
-        Patch.__init__(self, fill=False, **kwargs)
+        if 'fill' in kwargs:
+            super().__init__(**kwargs)
+        else:
+            fill = bool({'fc', 'facecolor', 'color'}.intersection(kwargs))
+            super().__init__(fill=fill, **kwargs)
         self.bbox1 = bbox1
         self.bbox2 = bbox2
         self.loc1 = loc1
         self.loc2 = loc2
 
     def get_path(self):
+        # docstring inherited
         return self.connect_bbox(self.bbox1, self.bbox2,
                                  self.loc1, self.loc2)
-    get_path.__doc__ = Patch.get_path.__doc__
 
 
 class BboxConnectorPatch(BboxConnector):
@@ -327,10 +298,10 @@ class BboxConnectorPatch(BboxConnector):
         """
         Connect two bboxes with a quadrilateral.
 
-        The quadrilateral is specified by two lines that start and end at corners
-        of the bboxes. The four sides of the quadrilateral are defined by the two
-        lines given, the line between the two corners specified in *bbox1* and the
-        line between the two corners specified in *bbox2*.
+        The quadrilateral is specified by two lines that start and end at
+        corners of the bboxes. The four sides of the quadrilateral are defined
+        by the two lines given, the line between the two corners specified in
+        *bbox1* and the line between the two corners specified in *bbox2*.
 
         Parameters
         ----------
@@ -357,42 +328,67 @@ class BboxConnectorPatch(BboxConnector):
 
         **kwargs
             Patch properties for the line drawn:
-            %(Patch)s
+
+            %(Patch:kwdoc)s
         """
         if "transform" in kwargs:
             raise ValueError("transform should not be set")
-        BboxConnector.__init__(self, bbox1, bbox2, loc1a, loc2a, **kwargs)
+        super().__init__(bbox1, bbox2, loc1a, loc2a, **kwargs)
         self.loc1b = loc1b
         self.loc2b = loc2b
 
     def get_path(self):
+        # docstring inherited
         path1 = self.connect_bbox(self.bbox1, self.bbox2, self.loc1, self.loc2)
         path2 = self.connect_bbox(self.bbox2, self.bbox1,
                                   self.loc2b, self.loc1b)
-        path_merged = (list(path1.vertices) +
-                       list(path2.vertices) +
-                       [path1.vertices[0]])
+        path_merged = [*path1.vertices, *path2.vertices, path1.vertices[0]]
         return Path(path_merged)
-    get_path.__doc__ = BboxConnector.get_path.__doc__
 
 
 def _add_inset_axes(parent_axes, inset_axes):
-    """Helper function to add an inset axes and disable navigation in it"""
+    """Helper function to add an inset axes and disable navigation in it."""
     parent_axes.figure.add_axes(inset_axes)
     inset_axes.set_navigate(False)
 
 
 @docstring.dedent_interpd
-def inset_axes(parent_axes, width, height, loc=1,
+def inset_axes(parent_axes, width, height, loc='upper right',
                bbox_to_anchor=None, bbox_transform=None,
-               axes_class=None,
-               axes_kwargs=None,
+               axes_class=None, axes_kwargs=None,
                borderpad=0.5):
     """
     Create an inset axes with a given width and height.
 
-    Both sizes used can be specified either in inches or percentage of the
-    parent axes.
+    Both sizes used can be specified either in inches or percentage.
+    For example,::
+
+        inset_axes(parent_axes, width='40%%', height='30%%', loc='lower left')
+
+    creates in inset axes in the lower left corner of *parent_axes* which spans
+    over 30%% in height and 40%% in width of the *parent_axes*. Since the usage
+    of `.inset_axes` may become slightly tricky when exceeding such standard
+    cases, it is recommended to read :doc:`the examples
+    </gallery/axes_grid1/inset_locator_demo>`.
+
+    Notes
+    -----
+    The meaning of *bbox_to_anchor* and *bbox_to_transform* is interpreted
+    differently from that of legend. The value of bbox_to_anchor
+    (or the return value of its get_points method; the default is
+    *parent_axes.bbox*) is transformed by the bbox_transform (the default
+    is Identity transform) and then interpreted as points in the pixel
+    coordinate (which is dpi dependent).
+
+    Thus, following three calls are identical and creates an inset axes
+    with respect to the *parent_axes*::
+
+       axins = inset_axes(parent_axes, "30%%", "40%%")
+       axins = inset_axes(parent_axes, "30%%", "40%%",
+                          bbox_to_anchor=parent_axes.bbox)
+       axins = inset_axes(parent_axes, "30%%", "40%%",
+                          bbox_to_anchor=(0, 0, 1, 1),
+                          bbox_transform=parent_axes.transAxes)
 
     Parameters
     ----------
@@ -400,59 +396,89 @@ def inset_axes(parent_axes, width, height, loc=1,
         Axes to place the inset axes.
 
     width, height : float or str
-        Size of the inset axes to create.
+        Size of the inset axes to create. If a float is provided, it is
+        the size in inches, e.g. *width=1.3*. If a string is provided, it is
+        the size in relative units, e.g. *width='40%%'*. By default, i.e. if
+        neither *bbox_to_anchor* nor *bbox_transform* are specified, those
+        are relative to the parent_axes. Otherwise they are to be understood
+        relative to the bounding box provided via *bbox_to_anchor*.
 
-    loc : int or string, optional, default to 1
-        Location to place the inset axes. The valid locations are::
-
-            'upper right'  : 1,
-            'upper left'   : 2,
-            'lower left'   : 3,
-            'lower right'  : 4,
-            'right'        : 5,
-            'center left'  : 6,
-            'center right' : 7,
-            'lower center' : 8,
-            'upper center' : 9,
-            'center'       : 10
+    loc : str, default: 'upper right'
+        Location to place the inset axes.  Valid locations are
+        'upper left', 'upper center', 'upper right',
+        'center left', 'center', 'center right',
+        'lower left', 'lower center, 'lower right'.
+        For backward compatibility, numeric values are accepted as well.
+        See the parameter *loc* of `.Legend` for details.
 
     bbox_to_anchor : tuple or `matplotlib.transforms.BboxBase`, optional
-        Bbox that the inset axes will be anchored. Can be a tuple of
-        [left, bottom, width, height], or a tuple of [left, bottom].
+        Bbox that the inset axes will be anchored to. If None,
+        a tuple of (0, 0, 1, 1) is used if *bbox_transform* is set
+        to *parent_axes.transAxes* or *parent_axes.figure.transFigure*.
+        Otherwise, *parent_axes.bbox* is used. If a tuple, can be either
+        [left, bottom, width, height], or [left, bottom].
+        If the kwargs *width* and/or *height* are specified in relative units,
+        the 2-tuple [left, bottom] cannot be used. Note that,
+        unless *bbox_transform* is set, the units of the bounding box
+        are interpreted in the pixel coordinate. When using *bbox_to_anchor*
+        with tuple, it almost always makes sense to also specify
+        a *bbox_transform*. This might often be the axes transform
+        *parent_axes.transAxes*.
 
     bbox_transform : `matplotlib.transforms.Transform`, optional
-        Transformation for the bbox. if None, `parent_axes.transAxes` is used.
+        Transformation for the bbox that contains the inset axes.
+        If None, a `.transforms.IdentityTransform` is used. The value
+        of *bbox_to_anchor* (or the return value of its get_points method)
+        is transformed by the *bbox_transform* and then interpreted
+        as points in the pixel coordinate (which is dpi dependent).
+        You may provide *bbox_to_anchor* in some normalized coordinate,
+        and give an appropriate transform (e.g., *parent_axes.transAxes*).
 
-    axes_class : `matplotlib.axes.Axes` type, optional
-        If specified, the inset axes created with be created with this class's
-        constructor.
+    axes_class : `matplotlib.axes.Axes` type, default: `.HostAxes`
+        The type of the newly created inset axes.
 
     axes_kwargs : dict, optional
-        Keyworded arguments to pass to the constructor of the inset axes.
+        Keyword arguments to pass to the constructor of the inset axes.
         Valid arguments include:
-        %(Axes)s
 
-    borderpad : float, optional
-        Padding between inset axes and the bbox_to_anchor. Defaults to 0.5.
+        %(Axes:kwdoc)s
+
+    borderpad : float, default: 0.5
+        Padding between inset axes and the bbox_to_anchor.
+        The units are axes font size, i.e. for a default font size of 10 points
+        *borderpad = 0.5* is equivalent to a padding of 5 points.
 
     Returns
     -------
-    inset_axes : `axes_class`
+    inset_axes : *axes_class*
         Inset axes object created.
     """
 
     if axes_class is None:
         axes_class = HostAxes
-
     if axes_kwargs is None:
-        inset_axes = axes_class(parent_axes.figure, parent_axes.get_position())
-    else:
-        inset_axes = axes_class(parent_axes.figure, parent_axes.get_position(),
-                                **axes_kwargs)
+        axes_kwargs = {}
+    inset_axes = axes_class(parent_axes.figure, parent_axes.get_position(),
+                            **axes_kwargs)
+
+    if bbox_transform in [parent_axes.transAxes,
+                          parent_axes.figure.transFigure]:
+        if bbox_to_anchor is None:
+            _api.warn_external("Using the axes or figure transform requires a "
+                               "bounding box in the respective coordinates. "
+                               "Using bbox_to_anchor=(0, 0, 1, 1) now.")
+            bbox_to_anchor = (0, 0, 1, 1)
 
     if bbox_to_anchor is None:
         bbox_to_anchor = parent_axes.bbox
 
+    if (isinstance(bbox_to_anchor, tuple) and
+            (isinstance(width, str) or isinstance(height, str))):
+        if len(bbox_to_anchor) != 4:
+            raise ValueError("Using relative units for width or height "
+                             "requires to provide a 4-tuple or a "
+                             "`Bbox` instance to `bbox_to_anchor.")
+
     axes_locator = AnchoredSizeLocator(bbox_to_anchor,
                                        width, height,
                                        loc=loc,
@@ -467,13 +493,13 @@ def inset_axes(parent_axes, width, height, loc=1,
 
 
 @docstring.dedent_interpd
-def zoomed_inset_axes(parent_axes, zoom, loc=1,
+def zoomed_inset_axes(parent_axes, zoom, loc='upper right',
                       bbox_to_anchor=None, bbox_transform=None,
-                      axes_class=None,
-                      axes_kwargs=None,
+                      axes_class=None, axes_kwargs=None,
                       borderpad=0.5):
     """
-    Create an anchored inset axes by scaling a parent axes.
+    Create an anchored inset axes by scaling a parent axes. For usage, also see
+    :doc:`the examples </gallery/axes_grid1/inset_locator_demo2>`.
 
     Parameters
     ----------
@@ -481,57 +507,66 @@ def zoomed_inset_axes(parent_axes, zoom, loc=1,
         Axes to place the inset axes.
 
     zoom : float
-        Scaling factor of the data axes. *zoom* > 1 will enlargen the
+        Scaling factor of the data axes. *zoom* > 1 will enlarge the
         coordinates (i.e., "zoomed in"), while *zoom* < 1 will shrink the
         coordinates (i.e., "zoomed out").
 
-    loc : int or string, optional, default to 1
-        Location to place the inset axes. The valid locations are::
-
-            'upper right'  : 1,
-            'upper left'   : 2,
-            'lower left'   : 3,
-            'lower right'  : 4,
-            'right'        : 5,
-            'center left'  : 6,
-            'center right' : 7,
-            'lower center' : 8,
-            'upper center' : 9,
-            'center'       : 10
+    loc : str, default: 'upper right'
+        Location to place the inset axes.  Valid locations are
+        'upper left', 'upper center', 'upper right',
+        'center left', 'center', 'center right',
+        'lower left', 'lower center, 'lower right'.
+        For backward compatibility, numeric values are accepted as well.
+        See the parameter *loc* of `.Legend` for details.
 
     bbox_to_anchor : tuple or `matplotlib.transforms.BboxBase`, optional
-        Bbox that the inset axes will be anchored. Can be a tuple of
-        [left, bottom, width, height], or a tuple of [left, bottom].
+        Bbox that the inset axes will be anchored to. If None,
+        *parent_axes.bbox* is used. If a tuple, can be either
+        [left, bottom, width, height], or [left, bottom].
+        If the kwargs *width* and/or *height* are specified in relative units,
+        the 2-tuple [left, bottom] cannot be used. Note that
+        the units of the bounding box are determined through the transform
+        in use. When using *bbox_to_anchor* it almost always makes sense to
+        also specify a *bbox_transform*. This might often be the axes transform
+        *parent_axes.transAxes*.
 
     bbox_transform : `matplotlib.transforms.Transform`, optional
-        Transformation for the bbox. if None, `parent_axes.transAxes` is used.
+        Transformation for the bbox that contains the inset axes.
+        If None, a `.transforms.IdentityTransform` is used (i.e. pixel
+        coordinates). This is useful when not providing any argument to
+        *bbox_to_anchor*. When using *bbox_to_anchor* it almost always makes
+        sense to also specify a *bbox_transform*. This might often be the
+        axes transform *parent_axes.transAxes*. Inversely, when specifying
+        the axes- or figure-transform here, be aware that not specifying
+        *bbox_to_anchor* will use *parent_axes.bbox*, the units of which are
+        in display (pixel) coordinates.
 
-    axes_class : `matplotlib.axes.Axes` type, optional
-        If specified, the inset axes created with be created with this class's
-        constructor.
+    axes_class : `matplotlib.axes.Axes` type, default: `.HostAxes`
+        The type of the newly created inset axes.
 
     axes_kwargs : dict, optional
-        Keyworded arguments to pass to the constructor of the inset axes.
+        Keyword arguments to pass to the constructor of the inset axes.
         Valid arguments include:
-        %(Axes)s
 
-    borderpad : float, optional
-        Padding between inset axes and the bbox_to_anchor. Defaults to 0.5.
+        %(Axes:kwdoc)s
+
+    borderpad : float, default: 0.5
+        Padding between inset axes and the bbox_to_anchor.
+        The units are axes font size, i.e. for a default font size of 10 points
+        *borderpad = 0.5* is equivalent to a padding of 5 points.
 
     Returns
     -------
-    inset_axes : `axes_class`
+    inset_axes : *axes_class*
         Inset axes object created.
     """
 
     if axes_class is None:
         axes_class = HostAxes
-
     if axes_kwargs is None:
-        inset_axes = axes_class(parent_axes.figure, parent_axes.get_position())
-    else:
-        inset_axes = axes_class(parent_axes.figure, parent_axes.get_position(),
-                                **axes_kwargs)
+        axes_kwargs = {}
+    inset_axes = axes_class(parent_axes.figure, parent_axes.get_position(),
+                            **axes_kwargs)
 
     axes_locator = AnchoredZoomLocator(parent_axes, zoom=zoom, loc=loc,
                                        bbox_to_anchor=bbox_to_anchor,
@@ -567,7 +602,8 @@ def mark_inset(parent_axes, inset_axes, loc1, loc2, **kwargs):
 
     **kwargs
         Patch properties for the lines and box drawn:
-        %(Patch)s
+
+        %(Patch:kwdoc)s
 
     Returns
     -------
@@ -579,7 +615,11 @@ def mark_inset(parent_axes, inset_axes, loc1, loc2, **kwargs):
     """
     rect = TransformedBbox(inset_axes.viewLim, parent_axes.transData)
 
-    pp = BboxPatch(rect, fill=False, **kwargs)
+    if 'fill' in kwargs:
+        pp = BboxPatch(rect, **kwargs)
+    else:
+        fill = bool({'fc', 'facecolor', 'color'}.intersection(kwargs))
+        pp = BboxPatch(rect, fill=fill, **kwargs)
     parent_axes.add_patch(pp)
 
     p1 = BboxConnector(inset_axes.bbox, rect, loc1=loc1, **kwargs)
